from dataclasses import dataclass


@dataclass(frozen=True)
class Radio:
    wifi_hw: bool
    wifi: bool
    wwan_hw: bool
    wwan: bool

    def to_json(self):
        return {
            'wifi_hw': self.wifi_hw,
            'wifi': self.wifi,
            'wwan_hw': self.wwan_hw,
            'wwan': self.wwan
        }



import re

from ._exception import UnspecifiedException
from ._system import SystemCommand, SystemCommandInterface
from .data.radio import Radio


class RadioControlInterface:

    def __call__(self) -> Radio:
        raise NotImplementedError

    def all(self) -> Radio:
        raise NotImplementedError

    def all_on(self) -> None:
        raise NotImplementedError

    def all_off(self) -> None:
        raise NotImplementedError

    def wifi(self) -> bool:
        raise NotImplementedError

    def wifi_on(self) -> None:
        raise NotImplementedError

    def wifi_off(self) -> None:
        raise NotImplementedError

    def wwan(self) -> bool:
        raise NotImplementedError

    def wwan_on(self) -> None:
        raise NotImplementedError

    def wwan_off(self) -> None:
        raise NotImplementedError


class RadioControl(RadioControlInterface):

    def __init__(self, syscmd: SystemCommandInterface = None):
        self._syscmd = syscmd or SystemCommand()

    def __call__(self) -> Radio:
        return self.all()

    def all(self) -> Radio:
        r = self._syscmd.nmcli(['radio', 'all'])
        row = r.split('\n')[1]
        m = re.search(r'^(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s*', row)
        if m:
            wifi_hw, wifi, wwan_hw, wwan = m.groups()
            return Radio(wifi_hw == 'enabled', wifi == 'enabled',
                         wwan_hw == 'enabled', wwan == 'enabled')
        raise UnspecifiedException(
            'Failed to parse the results of the radio command')

    def all_on(self) -> None:
        self._syscmd.nmcli(['radio', 'all', 'on'])

    def all_off(self) -> None:
        self._syscmd.nmcli(['radio', 'all', 'off'])

    def wifi(self) -> bool:
        r = self._syscmd.nmcli(['radio', 'wifi'])
        return r.replace('\n', '') == 'enabled'

    def wifi_on(self) -> None:
        self._syscmd.nmcli(['radio', 'wifi', 'on'])

    def wifi_off(self) -> None:
        self._syscmd.nmcli(['radio', 'wifi', 'off'])

    def wwan(self) -> bool:
        r = self._syscmd.nmcli(['radio', 'wwan'])
        return r.replace('\n', '') == 'enabled'

    def wwan_on(self) -> None:
        self._syscmd.nmcli(['radio', 'wwan', 'on'])

    def wwan_off(self) -> None:
        self._syscmd.nmcli(['radio', 'wwan', 'off'])



from typing import List, Tuple

from .._device import DeviceControlInterface, DeviceDetails
from ..data.device import Device, DeviceWifi
from ..data.hotspot import Hotspot


class DummyDeviceControl(DeviceControlInterface):

    @property
    def show_args(self):
        return self._show_args

    @property
    def connect_args(self):
        return self._connect_args

    @property
    def disconnect_args(self):
        return self._disconnect_args

    @property
    def reapply_args(self):
        return self._reapply_args

    @property
    def delete_args(self):
        return self._delete_args

    @property
    def wifi_args(self):
        return self._wifi_args

    @property
    def wifi_connect_args(self):
        return self._wifi_connect_args

    @property
    def wifi_hotspot_args(self):
        return self._wifi_hotspot_args

    @property
    def wifi_rescan_args(self):
        return self._wifi_rescan_args

    def __init__(self,
                 result_call: List[Device] = None,
                 result_show: DeviceDetails = None,
                 result_show_all: List[DeviceDetails] = None,
                 result_wifi: List[DeviceWifi] = None,
                 result_wifi_hotspot: Hotspot = None,
                 raise_error: Exception = None):
        self._raise_error = raise_error
        self._result_call = result_call or []
        self._result_wifi = result_wifi or []
        self._result_show = result_show
        self._result_show_all = result_show_all or []
        self._result_wifi_hotspot = result_wifi_hotspot
        self._show_args: List[Tuple] = []
        self._connect_args: List[Tuple] = []
        self._disconnect_args: List[Tuple] = []
        self._reapply_args: List[str] = []
        self._delete_args: List[Tuple] = []
        self._wifi_args: List[str] = []
        self._wifi_connect_args: List[Tuple] = []
        self._wifi_hotspot_args: List[Tuple] = []
        self._wifi_rescan_args: List[Tuple] = []

    def __call__(self) -> List[Device]:
        self._raise_error_if_needed()
        return self._result_call

    def status(self) -> List[Device]:
        self._raise_error_if_needed()
        return self._result_call

    def show(self, ifname: str, fields: str = None) -> DeviceDetails:
        self._raise_error_if_needed()
        self._show_args.append((ifname, fields))
        if not self._result_show is None:
            return self._result_show
        raise ValueError("'result_show' is not properly initialized")

    def show_all(self, fields: str = None) -> List[DeviceDetails]:
        self._raise_error_if_needed()
        return self._result_show_all

    def connect(self, ifname: str, wait: int = None) -> None:
        self._raise_error_if_needed()
        self._connect_args.append((ifname, wait))

    def disconnect(self, ifname: str, wait: int = None) -> None:
        self._raise_error_if_needed()
        self._disconnect_args.append((ifname, wait))

    def reapply(self, ifname: str) -> None:
        self._raise_error_if_needed()
        self._reapply_args.append(ifname)

    def delete(self, ifname: str, wait: int = None) -> None:
        self._raise_error_if_needed()
        self._delete_args.append((ifname, wait))

    def wifi(self, ifname: str = None) -> List[DeviceWifi]:
        self._raise_error_if_needed()
        if ifname is not None:
            self._wifi_args.append(ifname)
        return self._result_wifi

    def wifi_connect(self,
                     ssid: str,
                     password: str,
                     ifname: str = None,
                     wait: int = None) -> None:
        self._raise_error_if_needed()
        self._wifi_connect_args.append((ssid, password, ifname, wait))

    def wifi_hotspot(self,
                     ifname: str = None,
                     con_name: str = None,
                     ssid: str = None,
                     band: str = None,
                     channel: int = None,
                     password: str = None) -> Hotspot:
        self._raise_error_if_needed()
        self._wifi_hotspot_args.append(
            (ifname, con_name, ssid, band, channel, password))
        if not self._result_wifi_hotspot is None:
            return self._result_wifi_hotspot
        raise ValueError("'result_wifi_hotspot' is not properly initialized")

    def wifi_rescan(self,
                    ifname: str = None,
                    ssid: str = None) -> None:
        self._raise_error_if_needed()
        self._wifi_rescan_args.append((ifname, ssid))

    def _raise_error_if_needed(self):
        if not self._raise_error is None:
            raise self._raise_error



import re
from typing import List, Tuple

from ._exception import ConnectionActivateFailedException
from ._helper import add_fields_option_if_needed, add_wait_option_if_needed
from ._system import SystemCommand, SystemCommandInterface
from .data.device import Device, DeviceDetails, DeviceWifi
from .data.hotspot import Hotspot


def parse_wifi_hotspot_result(text: str) -> Tuple[str, str, str]:
    m = re.search(r'Hotspot\spassword:\s(.*)', text)
    if m:
        password = m.groups()[0]
    m = re.search(
        r"Device\s'(.*)'\ssuccessfully\sactivated\swith\s'(.*)'", text)
    if m:
        ifname, uuid = m.groups()
    return (uuid, ifname, password)


def parse_hotspot_properties(text: str) -> Tuple[str, str]:
    for row in text.split('\n'):
        if len(row) == 0:
            continue
        m = re.search(r'^([\w\.\-]+):([\S\s]+)', row)
        if m is None:
            continue
        key, value = m.groups()
        if key == 'connection.id':
            con_name = value
        elif key == '802-11-wireless.ssid':
            ssid = value
    return (con_name, ssid)


class DeviceControlInterface:

    def __call__(self) -> List[Device]:
        raise NotImplementedError

    def status(self) -> List[Device]:
        raise NotImplementedError

    def show(self, ifname: str, fields: str = None) -> DeviceDetails:
        raise NotImplementedError

    def show_all(self, fields: str = None) -> List[DeviceDetails]:
        raise NotImplementedError

    def connect(self, ifname: str, wait: int = None) -> None:
        raise NotImplementedError

    def disconnect(self, ifname: str, wait: int = None) -> None:
        raise NotImplementedError

    def reapply(self, ifname: str) -> None:
        raise NotImplementedError

    def delete(self, ifname: str, wait: int = None) -> None:
        raise NotImplementedError

    def wifi(self, ifname: str = None) -> List[DeviceWifi]:
        raise NotImplementedError

    def wifi_connect(self,
                     ssid: str,
                     password: str,
                     ifname: str = None,
                     wait: int = None) -> None:
        raise NotImplementedError

    def wifi_hotspot(self,
                     ifname: str = None,
                     con_name: str = None,
                     ssid: str = None,
                     band: str = None,
                     channel: int = None,
                     password: str = None) -> Hotspot:
        raise NotImplementedError

    def wifi_rescan(self,
                    ifname: str = None,
                    ssid: str = None) -> None:
        raise NotImplementedError


class DeviceControl(DeviceControlInterface):

    def __init__(self, syscmd: SystemCommandInterface = None):
        self._syscmd = syscmd or SystemCommand()

    def __call__(self) -> List[Device]:
        return self.status()

    def status(self) -> List[Device]:
        r = self._syscmd.nmcli(['device', 'status'])
        results = []
        for row in r.split('\n')[1:]:
            if len(row) > 0:
                results.append(Device.parse(row))
        return results

    def show(self, ifname: str, fields: str = None) -> DeviceDetails:
        cmd = add_fields_option_if_needed(fields) + ['device', 'show', ifname]
        r = self._syscmd.nmcli(cmd)
        results = {}
        for row in r.split('\n'):
            m = re.search(r'^(\S+):\s*([\S\s]+)\s*', row)
            if m:
                key, value = m.groups()
                results[key] = None if value in ('--', '""') else value
        return results

    def show_all(self, fields: str = None) -> List[DeviceDetails]:
        cmd = add_fields_option_if_needed(fields) + ['device', 'show']
        r = self._syscmd.nmcli(cmd)
        results = []
        details: DeviceDetails = {}
        for row in r.split('\n'):
            m = re.search(r'^(\S+):\s*([\S\s]+)\s*', row)
            if m:
                key, value = m.groups()
                if key == 'GENERAL.DEVICE':
                    details = {}
                    results.append(details)
                details[key] = None if value in ('--', '""') else value
        return results

    def connect(self, ifname: str, wait: int = None) -> None:
        cmd = add_wait_option_if_needed(
            wait) + ['device', 'connect', ifname]
        self._syscmd.nmcli(cmd)

    def disconnect(self, ifname: str, wait: int = None) -> None:
        cmd = add_wait_option_if_needed(
            wait) + ['device', 'disconnect', ifname]
        self._syscmd.nmcli(cmd)

    def reapply(self, ifname: str) -> None:
        self._syscmd.nmcli(['device', 'reapply', ifname])

    def delete(self, ifname: str, wait: int = None) -> None:
        cmd = add_wait_option_if_needed(
            wait) + ['device', 'delete', ifname]
        self._syscmd.nmcli(cmd)

    def wifi(self, ifname: str = None) -> List[DeviceWifi]:
        cmd = ['-t', '-f', 'IN-USE,SSID,BSSID,MODE,CHAN,FREQ,RATE,SIGNAL,SECURITY',
               'device', 'wifi', 'list']
        if ifname is not None:
            cmd += ['ifname', ifname]
        r = self._syscmd.nmcli(cmd)
        results = []
        rows = r.split('\n')
        for row in rows:
            if len(row) > 0:
                results.append(DeviceWifi.parse(row))
        return results

    def wifi_connect(self,
                     ssid: str,
                     password: str,
                     ifname: str = None,
                     wait: int = None) -> None:
        cmd = add_wait_option_if_needed(
            wait) + ['device', 'wifi', 'connect', ssid, 'password', password]
        if ifname is not None:
            cmd += ['ifname', ifname]
        r = self._syscmd.nmcli(cmd)
        m = re.search(r'Connection activation failed:', r)
        if m:
            raise ConnectionActivateFailedException(
                'Connection activation failed')

    def wifi_hotspot(self,
                     ifname: str = None,
                     con_name: str = None,
                     ssid: str = None,
                     band: str = None,
                     channel: int = None,
                     password: str = None) -> Hotspot:
        cmd = ['device', 'wifi', 'hotspot', '--show-secrets']
        if ifname is not None:
            cmd += ['ifname', ifname]
        if con_name is not None:
            cmd += ['con-name', con_name]
        if ssid is not None:
            cmd += ['ssid', ssid]
        if band is not None:
            cmd += ['band', band]
        if channel is not None:
            cmd += ['channel', str(channel)]
        if password is not None:
            cmd += ['password', password]
        r = self._syscmd.nmcli(cmd)
        uuid, ifname, password = parse_wifi_hotspot_result(r)
        r = self._syscmd.nmcli(['-t', '-f', 'connection.id,802-11-wireless.ssid',
                                'connection', 'show', 'uuid', uuid])
        con_name, ssid = parse_hotspot_properties(r)
        return Hotspot(ifname, con_name, ssid, password)

    def wifi_rescan(self,
                    ifname: str = None,
                    ssid: str = None) -> None:
        cmd = ['device', 'wifi', 'rescan']
        if ifname is not None:
            cmd += ['ifname', ifname]
        if ssid is not None:
            cmd += ['ssid', ssid]
        self._syscmd.nmcli(cmd)



from ._connection import DummyConnectionControl
from ._device import DummyDeviceControl
from ._general import DummyGeneralControl
from ._networking import DummyNetworkingControl
from ._radio import DummyRadioControl



import re
from typing import List, Optional

from ._helper import add_wait_option_if_needed
from ._system import SystemCommand, SystemCommandInterface
from .data.connection import Connection, ConnectionDetails, ConnectionOptions


class ConnectionControlInterface:

    def __call__(self) -> List[Connection]:
        raise NotImplementedError

    def add(self,
            conn_type: str,
            options: Optional[ConnectionOptions] = None,
            ifname: str = "*",
            name: str = None,
            autoconnect: bool = None) -> None:
        raise NotImplementedError

    def modify(self, name: str, options: ConnectionOptions) -> None:
        raise NotImplementedError

    def delete(self, name: str, wait: int = None) -> None:
        raise NotImplementedError

    def up(self, name: str, wait: int = None) -> None:
        raise NotImplementedError

    def down(self, name: str, wait: int = None) -> None:
        raise NotImplementedError

    def show(self, name: str) -> ConnectionDetails:
        raise NotImplementedError

    def reload(self) -> None:
        raise NotImplementedError


class ConnectionControl(ConnectionControlInterface):

    def __init__(self, syscmd: SystemCommandInterface = None):
        self._syscmd = syscmd or SystemCommand()

    def __call__(self) -> List[Connection]:
        r = self._syscmd.nmcli('connection')
        results = []
        for row in r.split('\n')[1:]:
            if len(row) == 0:
                continue
            results.append(Connection.parse(row))
        return results

    def add(self,
            conn_type: str,
            options: Optional[ConnectionOptions] = None,
            ifname: str = "*",
            name: str = None,
            autoconnect: bool = None) -> None:
        cmd = ['connection', 'add', 'type', conn_type, 'ifname', ifname]
        if autoconnect is not None:
            cmd += ['autoconnect', 'yes' if autoconnect else 'no']
        if not name is None:
            cmd += ['con-name', name]
        options = {} if options is None else options
        for k, v in options.items():
            cmd += [k, v]
        self._syscmd.nmcli(cmd)

    def modify(self, name: str, options: ConnectionOptions) -> None:
        cmd = ['connection', 'modify', name]
        for k, v in options.items():
            cmd += [k, v]
        self._syscmd.nmcli(cmd)

    def delete(self, name: str, wait: int = None) -> None:
        cmd = add_wait_option_if_needed(
            wait) + ['connection', 'delete', name]
        self._syscmd.nmcli(cmd)

    def up(self, name: str, wait: int = None) -> None:
        cmd = add_wait_option_if_needed(wait) + ['connection', 'up', name]
        self._syscmd.nmcli(cmd)

    def down(self, name: str, wait: int = None) -> None:
        cmd = add_wait_option_if_needed(
            wait) + ['connection', 'down', name]
        self._syscmd.nmcli(cmd)

    def show(self, name: str) -> ConnectionDetails:
        r = self._syscmd.nmcli(['connection', 'show', name])
        results = {}
        for row in r.split('\n'):
            m = re.search(r'^(\S+):\s*([\S\s]+)\s*', row)
            if m:
                key, value = m.groups()
                results[key] = None if value in ('--', '""') else value
        return results

    def reload(self) -> None:
        self._syscmd.nmcli(['connection', 'reload'])



from ._connection import ConnectionControl, ConnectionControlInterface
from ._const import NetworkConnectivity, NetworkManagerState
from ._device import DeviceControl, DeviceControlInterface
from ._exception import (ConnectionActivateFailedException,
                         ConnectionDeactivateFailedException,
                         ConnectionDeleteFailedException,
                         DisconnectDeviceFailedException,
                         InvalidUserInputException,
                         NetworkManagerNotRunningException, NotExistException,
                         ScanningNotAllowedException, TimeoutExpiredException,
                         UnspecifiedException)
from ._general import GeneralControl, GeneralControlInterface
from ._networking import NetworkingControl, NetworkingControlInterface
from ._radio import RadioControl, RadioControlInterface
from ._system import CommandParameter, SystemCommand, SystemCommandInterface
from .data import (Connection, ConnectionDetails, ConnectionOptions, Device,
                   DeviceDetails, DeviceWifi, General, Radio)

_syscmd = SystemCommand()
connection = ConnectionControl(_syscmd)
device = DeviceControl(_syscmd)
general = GeneralControl(_syscmd)
networking = NetworkingControl(_syscmd)
radio = RadioControl(_syscmd)


def disable_use_sudo():
    _syscmd.disable_use_sudo()


def set_lang(lang: str):
    _syscmd.set_lang(lang)



from subprocess import CalledProcessError, run
from typing import List, Union

from ._exception import (ConnectionActivateFailedException,
                         ConnectionDeactivateFailedException,
                         ConnectionDeleteFailedException,
                         DisconnectDeviceFailedException,
                         InvalidUserInputException,
                         NetworkManagerNotRunningException, NotExistException,
                         ScanningNotAllowedException, TimeoutExpiredException,
                         UnspecifiedException)

CommandParameter = Union[str, List[str]]


class SystemCommandInterface:

    def nmcli(self, parameters: CommandParameter) -> str:
        raise NotImplementedError

    def disable_use_sudo(self):
        raise NotImplementedError

    def set_lang(self, lang: str):
        raise NotImplementedError


class SystemCommand(SystemCommandInterface):

    def __init__(self, subprocess_run=run):
        self._run = subprocess_run
        self._use_sudo = True
        self._lang = 'C'

    def nmcli(self, parameters: CommandParameter) -> str:
        if isinstance(parameters, str):
            parameters = [parameters]
        c = ['sudo', 'nmcli'] if self._use_sudo else ['nmcli']
        commands = c + parameters
        try:
            r = self._run(commands, capture_output=True,
                          check=True, env={'LANG': self._lang})
            return r.stdout.decode('utf-8')
        except CalledProcessError as e:
            rc = e.returncode
            stderr = e.stderr.decode('utf-8')
            if rc == 2:
                raise InvalidUserInputException(
                    'Invalid user input, wrong nmcli invocation') from e
            elif rc == 3:
                raise TimeoutExpiredException('Timeout expired') from e
            elif rc == 4:
                raise ConnectionActivateFailedException(
                    'Connection activation failed') from e
            elif rc == 5:
                raise ConnectionDeactivateFailedException(
                    'Connection deactivation failed') from e
            elif rc == 6:
                raise DisconnectDeviceFailedException(
                    'Disconnecting device failed') from e
            elif rc == 7:
                raise ConnectionDeleteFailedException(
                    'Connection deletion failed') from e
            elif rc == 8:
                raise NetworkManagerNotRunningException(
                    'NetworkManager is not running') from e
            elif rc == 10:
                raise NotExistException(
                    'Connection, device, or access point does not exist') from e
            else:
                if rc == 1 and stderr.find('Scanning not allowed') > 0:
                    raise ScanningNotAllowedException(stderr) from e
                raise UnspecifiedException(
                    f'Unknown or unspecified error [code:{rc}, detail:{stderr}]') from e

    def disable_use_sudo(self):
        self._use_sudo = False

    def set_lang(self, lang: str):
        self._lang = lang



from typing import List

from .._const import NetworkConnectivity
from .._networking import NetworkingControlInterface


class DummyNetworkingControl(NetworkingControlInterface):

    @property
    def called_on(self) -> int:
        return self._called_on

    @property
    def called_off(self) -> int:
        return self._called_off

    @property
    def connectivity_args(self) -> List[bool]:
        return self._connectivity_args

    def __init__(self,
                 result_call: NetworkConnectivity = None,
                 result_connectivity: NetworkConnectivity = None,
                 raise_error: Exception = None):
        self._result_call = result_call
        self._result_connectivity = result_connectivity
        self._raise_error = raise_error
        self._called_on = 0
        self._called_off = 0
        self._connectivity_args: List[bool] = []

    def __call__(self) -> NetworkConnectivity:
        self._raise_error_if_needed()
        if not self._result_call is None:
            return self._result_call
        raise ValueError("'result_call' is not properly initialized")

    def on(self) -> None:
        self._raise_error_if_needed()
        self._called_on += 1

    def off(self) -> None:
        self._raise_error_if_needed()
        self._called_off += 1

    def connectivity(self, check: bool = False) -> NetworkConnectivity:
        self._raise_error_if_needed()
        self._connectivity_args.append(check)
        if not self._result_connectivity is None:
            return self._result_connectivity
        raise ValueError("'result_connectivity' is not properly initialized")

    def _raise_error_if_needed(self):
        if not self._raise_error is None:
            raise self._raise_error



class UnspecifiedException(Exception):
    """
    1 Unknown or unspecified error.
    """


class InvalidUserInputException(Exception):
    """
    2 Invalid user input, wrong nmcli invocation.
    """


class TimeoutExpiredException(Exception):
    """
    3 Timeout expired.
    """


class ConnectionActivateFailedException(Exception):
    """
    4 Connection activation failed.
    """


class ConnectionDeactivateFailedException(Exception):
    """
    5 Connection deactivation failed.
    """


class DisconnectDeviceFailedException(Exception):
    """
    6 Disconnecting device failed.
    """


class ConnectionDeleteFailedException(Exception):
    """
    7 Connection deletion failed.
    """


class NetworkManagerNotRunningException(Exception):
    """
    8 NetworkManager is not running.
    """


class NotExistException(Exception):
    """
    10 Connection, device, or access point does not exist.
    """


class ScanningNotAllowedException(Exception):
    """
    Scanning not allowed.
    """



from __future__ import annotations

import re
from dataclasses import dataclass
from typing import Dict, Optional

ConnectionDetails = Dict[str, Optional[str]]
ConnectionOptions = Dict[str, str]


@dataclass(frozen=True)
class Connection:
    name: str
    uuid: str
    conn_type: str
    device: str

    def to_json(self):
        return {
            'name': self.name,
            'uuid': self.uuid,
            'conn_type': self.conn_type,
            'device': self.device
        }

    @classmethod
    def parse(cls, text: str) -> Connection:
        m = re.search(r'^([\S\s]+)\s{2}(\S+)\s{2}(\S+)\s+(\S+)\s*', text)
        if m:
            name, uuid, conn_type, device = m.groups()
            return Connection(name.strip(), uuid, conn_type, device)
        raise ValueError(f'Parse failed [{text}]')



from enum import Enum


class NetworkManagerState(Enum):
    UNKNOWN = 'unknown'
    ASLEEP = 'asleep'
    CONNECTING = 'connecting'
    CONNECTED_LOCAL = 'connected (local only)'
    CONNECTED_SITE = 'connected (site only)'
    CONNECTED_GLOBAL = 'connected'
    DISCONNECTING = 'disconnecting'
    DISCONNECTED = 'disconnected'


class NetworkConnectivity(Enum):
    UNKNOWN = 'unknown'
    NONE = 'none'
    PORTAL = 'portal'
    LIMITED = 'limited'
    FULL = 'full'



from __future__ import annotations

import re
from dataclasses import dataclass

from .._const import NetworkConnectivity, NetworkManagerState


@dataclass(frozen=True)
class General:
    state: NetworkManagerState
    connectivity: NetworkConnectivity
    wifi_hw: bool
    wifi: bool
    wwan_hw: bool
    wwan: bool

    def to_json(self):
        return {
            'state': self.state.value,
            'connectivity': self.connectivity.value,
            'wifi_hw': self.wifi_hw,
            'wifi': self.wifi,
            'wwan_hw': self.wwan_hw,
            'wwan': self.wwan
        }

    @classmethod
    def parse(cls, text: str) -> General:
        m = re.search(
            r'^([\S\s]+)\s{2}(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s*', text)
        if m:
            state, connectivity, wifi_hw, wifi, wwan_hw, wwan = m.groups()
            return General(NetworkManagerState(state),
                           NetworkConnectivity(connectivity),
                           wifi_hw == 'enabled',
                           wifi == 'enabled',
                           wwan_hw == 'enabled',
                           wwan == 'enabled')
        raise ValueError(f'Parse failed [{text}]')



from .connection import Connection, ConnectionDetails, ConnectionOptions
from .device import Device, DeviceDetails, DeviceWifi
from .general import General
from .hotspot import Hotspot
from .radio import Radio



from dataclasses import dataclass


@dataclass(frozen=True)
class Hotspot:
    ifname: str
    con_name: str
    ssid: str
    password: str

    def to_json(self):
        return {
            'ifname': self.ifname,
            'con_name': self.con_name,
            'ssid': self.ssid,
            'password': self.password
        }



from ._const import NetworkConnectivity
from ._system import SystemCommand, SystemCommandInterface


class NetworkingControlInterface:

    def __call__(self) -> NetworkConnectivity:
        raise NotImplementedError

    def on(self) -> None:
        raise NotImplementedError

    def off(self) -> None:
        raise NotImplementedError

    def connectivity(self, check: bool = False) -> NetworkConnectivity:
        raise NotImplementedError


class NetworkingControl(NetworkingControlInterface):

    def __init__(self, syscmd: SystemCommandInterface = None):
        self._syscmd = syscmd or SystemCommand()

    def __call__(self) -> NetworkConnectivity:
        return self.connectivity()

    def on(self) -> None:
        self._syscmd.nmcli(['networking', 'on'])

    def off(self) -> None:
        self._syscmd.nmcli(['networking', 'off'])

    def connectivity(self, check: bool = False) -> NetworkConnectivity:
        cmds = ['networking', 'connectivity']
        if check:
            cmds += ['check']
        return NetworkConnectivity(self._syscmd.nmcli(cmds).replace('\n', ''))



from typing import List, Optional, Tuple

from .._connection import (ConnectionControlInterface, ConnectionDetails,
                           ConnectionOptions)
from ..data.connection import Connection


class DummyConnectionControl(ConnectionControlInterface):

    @property
    def add_args(self):
        return self._add_args

    @property
    def modify_args(self):
        return self._modify_args

    @property
    def delete_args(self):
        return self._delete_args

    @property
    def up_args(self):
        return self._up_args

    @property
    def down_args(self):
        return self._down_args

    @property
    def called_reload(self) -> int:
        return self._called_reload

    def __init__(self,
                 result_call: List[Connection] = None,
                 result_show: ConnectionDetails = None,
                 raise_error: Exception = None):
        self._raise_error = raise_error
        self._result_call = result_call or []
        self._result_show = result_show
        self._add_args: List[Tuple] = []
        self._modify_args: List[Tuple] = []
        self._delete_args: List[Tuple] = []
        self._up_args: List[Tuple] = []
        self._down_args: List[Tuple] = []
        self._called_reload = 0

    def __call__(self) -> List[Connection]:
        self._raise_error_if_needed()
        return self._result_call

    def add(self,
            conn_type: str,
            options: Optional[ConnectionOptions] = None,
            ifname: str = "*",
            name: str = None,
            autoconnect: bool = None) -> None:
        self._raise_error_if_needed()
        self._add_args.append((conn_type, options, ifname, name, autoconnect))

    def modify(self, name: str, options: ConnectionOptions) -> None:
        self._raise_error_if_needed()
        self._modify_args.append((name, options))

    def delete(self, name: str, wait: int = None) -> None:
        self._raise_error_if_needed()
        self._delete_args.append((name, wait))

    def up(self, name: str, wait: int = None) -> None:
        self._raise_error_if_needed()
        self._up_args.append((name, wait))

    def down(self, name: str, wait: int = None) -> None:
        self._raise_error_if_needed()
        self._down_args.append((name, wait))

    def show(self, name: str) -> ConnectionDetails:
        self._raise_error_if_needed()
        if not self._result_show is None:
            return self._result_show
        raise ValueError("'result_show' is not properly initialized")

    def reload(self) -> None:
        self._raise_error_if_needed()
        self._called_reload += 1

    def _raise_error_if_needed(self):
        if not self._raise_error is None:
            raise self._raise_error



#!/usr/bin/env python3
from setuptools import setup
from setuptools.command.test import test

class PyTest(test):

    def run_tests(self):
        import pytest
        pytest.main(self.test_args)

with open('README.md', 'r') as f:
    long_description = f.read()

setup(
    name='nmcli',
    version='1.1.0',
    author='ushiboy',
    license='MIT',
    description='A python wrapper library for the network-manager cli client',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/ushiboy/nmcli',
    packages=['nmcli', 'nmcli.data', 'nmcli.dummy'],
    package_data={
        'nmcli': ['py.typed'],
    },
    test_suite='tests',
    python_requires='>=3.7',
    tests_require=[
        'pytest'
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX :: Linux'
    ],
    cmdclass={'test': PyTest}
)



from typing import List

from .._general import GeneralControlInterface
from ..data.general import General


class DummyGeneralControl(GeneralControlInterface):

    @property
    def set_hostname_args(self):
        return self._set_hostname_args

    def __init__(self,
                 result_call: General = None,
                 result_status: General = None,
                 result_hostname: str = 'localhost',
                 raise_error: Exception = None):
        self._raise_error = raise_error
        self._result_call = result_call
        self._result_status = result_status
        self._result_hostname = result_hostname
        self._set_hostname_args: List[str] = []

    def __call__(self) -> General:
        self._raise_error_if_needed()
        if not self._result_call is None:
            return self._result_call
        raise ValueError("'result_call' is not properly initialized")

    def status(self) -> General:
        self._raise_error_if_needed()
        if not self._result_status is None:
            return self._result_status
        raise ValueError("'result_status' is not properly initialized")

    def get_hostname(self) -> str:
        self._raise_error_if_needed()
        return self._result_hostname

    def set_hostname(self, hostname: str):
        self._raise_error_if_needed()
        self._set_hostname_args.append(hostname)

    def _raise_error_if_needed(self):
        if not self._raise_error is None:
            raise self._raise_error



# pylint: disable=line-too-long
from __future__ import annotations

import re
from dataclasses import dataclass
from typing import Dict, Optional

DeviceDetails = Dict[str, Optional[str]]


@dataclass(frozen=True)
class Device:
    device: str
    device_type: str
    state: str
    connection: Optional[str]

    def to_json(self):
        return {
            'device': self.device,
            'device_type': self.device_type,
            'state': self.state,
            'connection': self.connection
        }

    @classmethod
    def parse(cls, text: str) -> Device:
        m = re.search(r'^(\S*)\s+(\S*)\s+(\S*)\s+([\S\s]+)\s*$', text)
        if m:
            device, device_type, state, conn = m.groups()
            conn = conn.strip()
            connection = conn if conn != '--' else None
            return Device(device, device_type, state, connection)
        raise ValueError(f'Parse failed [{text}]')


@dataclass(frozen=True)
class DeviceWifi:
    in_use: bool
    ssid: str
    bssid: str
    mode: str
    chan: int
    freq: int
    rate: int
    signal: int
    security: str

    def to_json(self):
        return {
            'in_use': self.in_use,
            'ssid': self.ssid,
            'bssid': self.bssid,
            'mode': self.mode,
            'chan': self.chan,
            'freq': self.freq,
            'rate': self.rate,
            'signal': self.signal,
            'security': self.security
        }

    @classmethod
    def parse(cls, text: str) -> DeviceWifi:
        t = text.replace("\\:", "\uFFFE").replace(
            ":", "\uFFFF").replace("\uFFFE", ":")
        m = re.search(
            r'^(\*|\s)\uFFFF(.*)\uFFFF(.*)\uFFFF(.*)\uFFFF(\d+)\uFFFF(\d+)\sMHz\uFFFF(\d+)\sMbit\/s\uFFFF(\d+)\uFFFF(.*)$', t)
        if m:
            in_use, ssid, bssid, mode, chan, freq, rate, signal, security = m.groups()
            return DeviceWifi(in_use == '*', ssid, bssid, mode,
                              int(chan), int(freq), int(rate), int(signal), security.rstrip())
        raise ValueError(f'Parse failed [{text}]')



from ._system import SystemCommand, SystemCommandInterface
from .data import General


class GeneralControlInterface:

    def __call__(self) -> General:
        raise NotImplementedError

    def status(self) -> General:
        raise NotImplementedError

    def get_hostname(self) -> str:
        raise NotImplementedError

    def set_hostname(self, hostname: str):
        raise NotImplementedError


class GeneralControl(GeneralControlInterface):

    def __init__(self, syscmd: SystemCommandInterface = None):
        self._syscmd = syscmd or SystemCommand()

    def __call__(self) -> General:
        return self.status()

    def status(self) -> General:
        r = self._syscmd.nmcli(['general', 'status'])
        return General.parse(r.split('\n')[1])

    def get_hostname(self) -> str:
        r = self._syscmd.nmcli(['general', 'hostname'])
        return r.replace('\n', '')

    def set_hostname(self, hostname: str):
        self._syscmd.nmcli(['general', 'hostname', hostname])



from typing import List


def add_wait_option_if_needed(wait: int = None) -> List[str]:
    return [] if wait is None else ['--wait', str(wait)]


def add_fields_option_if_needed(fields: str = None) -> List[str]:
    return [] if fields is None else ['-f', fields]



from .._radio import RadioControlInterface
from ..data.radio import Radio


class DummyRadioControl(RadioControlInterface):

    @property
    def called_all_on(self) -> int:
        return self._called_all_on

    @property
    def called_all_off(self) -> int:
        return self._called_all_off

    @property
    def called_wifi_on(self) -> int:
        return self._called_wifi_on

    @property
    def called_wifi_off(self) -> int:
        return self._called_wifi_off

    @property
    def called_wwan_on(self) -> int:
        return self._called_wwan_on

    @property
    def called_wwan_off(self) -> int:
        return self._called_wwan_off

    def __init__(self,
                 result_call: Radio = None,
                 result_all: Radio = None,
                 result_wifi: bool = None,
                 result_wwan: bool = None,
                 raise_error: Exception = None):
        self._result_call = result_call
        self._result_all = result_all
        self._result_wifi = result_wifi
        self._result_wwan = result_wwan
        self._raise_error = raise_error
        self._called_all_on = 0
        self._called_all_off = 0
        self._called_wifi_on = 0
        self._called_wifi_off = 0
        self._called_wwan_on = 0
        self._called_wwan_off = 0

    def __call__(self) -> Radio:
        self._raise_error_if_needed()
        if not self._result_call is None:
            return self._result_call
        raise ValueError("'result_call' is not properly initialized")

    def all(self) -> Radio:
        self._raise_error_if_needed()
        if not self._result_all is None:
            return self._result_all
        raise ValueError("'result_all' is not properly initialized")

    def all_on(self) -> None:
        self._raise_error_if_needed()
        self._called_all_on += 1

    def all_off(self) -> None:
        self._raise_error_if_needed()
        self._called_all_off += 1

    def wifi(self) -> bool:
        self._raise_error_if_needed()
        if not self._result_wifi is None:
            return self._result_wifi
        raise ValueError("'result_wifi' is not properly initialized")

    def wifi_on(self) -> None:
        self._raise_error_if_needed()
        self._called_wifi_on += 1

    def wifi_off(self) -> None:
        self._raise_error_if_needed()
        self._called_wifi_off += 1

    def wwan(self) -> bool:
        self._raise_error_if_needed()
        if not self._result_wwan is None:
            return self._result_wwan
        raise ValueError("'result_wwan' is not properly initialized")

    def wwan_on(self) -> None:
        self._raise_error_if_needed()
        self._called_wwan_on += 1

    def wwan_off(self) -> None:
        self._raise_error_if_needed()
        self._called_wwan_off += 1

    def _raise_error_if_needed(self):
        if not self._raise_error is None:
            raise self._raise_error
